#!/bin/bash
# $Id: doit.sh 35 2009-03-15 16:07:53Z lrupp $
#
# Create a openSUSE-Education DVD with this script
# needs inst-source-utils to be installed, first
#

PATH=~/bin:$PATH
CONFIG=/etc/sysconfig/opensuse-education
DIST="11.0"
DO_ECHO=""
RESIGN="false"
VERBOSE="false"
SYNC_REPOS="false"
LOGFILE="/tmp/opensuse-education.log"
MKISO="false"
CREATE_LINKS="false"

function usage (){
    echo
    echo "Usage: $0 [OPTIONS]"
    echo "       -c <config> : use this config instead of the default one ($CONFIG)"
    echo "       -d          ; debug - don't do it really, just print out commands"
    echo "       -l <logfile>: use logfile"
    echo "       -m          : create an ISO file"
    echo "       -y          : create symlinks"
    echo "       -r          : resign RPMs (overwrites config)"
    echo "       -s          : Sync the repositories"
    echo "       -v          : verbose output"
    echo 
    exit $1
}

function LOG (){
    LOG_DATE=`date "+%b %d %H:%M:%S"`
    HOST=`hostname`
    echo "$LOG_DATE $HOST [doit]: $1" >> $LOGFILE
    if [ "$VERBOSE" = "true" ]; then
        echo $1
    fi
}

while getopts 'c:dl:hrsv' OPTION; do
    case $OPTION in
        c) CONFIG="$OPTARG"
        ;;
        d) DO_ECHO="echo " 
           VERBOSE="true"
        ;;
        l) LOGFILE="$OPTARG"
        ;;
        h) usage "0"
        ;;
        r) RESIGN="true"
        ;;
        s) SYNC_REPOS="true"
        ;;
        v) VERBOSE="true"
        ;;
        *) usage "1"
    esac
done
shift $(( OPTIND - 1 ))

test -f "$CONFIG" || {
    echo "$CONFIG is no file"
    exit 1
}
. "$CONFIG"


if [ ! -d "$REPO_DIR/$DIST" ]; then 
    LOG "INFO:    Creating $REPO_DIR/$DIST"
    $DO_ECHO mkdir -p "$REPO_DIR/$DIST" 1>/dev/null
fi

if [ "$SYNC_REPOS" = "true" ]; then
    LOG "INFO:    Starting sync of repositories"
    if [ "$CLEAN_REPO_DIR" = "true" ]; then
        LOG "WARNING: Cleaning $REPO_DIR/$DIST"
        test -d "$REPO_DIR/$DIST" && $DO_ECHO rm -rf "$REPO_DIR/$DIST"
        test -d "$REPO_DIR/$DIST" || $DO_ECHO mkdir -p "$REPO_DIR/$DIST" 
    fi
    if [ -n "$REPO_FILE" ]; then
        if [ -f "$REPO_FILE" ]; then
          URLS=$(cat "$REPO_FILE" | grep ^baseurl= | cut -d "=" -f2)
          for URL in $URLS; do
            if [ "$DO_ECHO" = "" ]; then
                LOG "INFO:    Syncing $URL"
                rsync_repos.sh -c "$CONFIG" -d "$DIST" -l "$LOGFILE" -s "$DOWNLOAD_DIR" -u $URL
            else
                echo "Calling rsync_repos.sh -v -c \"$CONFIG\" -d \"$DIST\" -l \"$LOGFILE\" -s \"$DOWNLOAD_DIR\" -u $URL"
                rsync_repos.sh -v -c "$CONFIG" -d "$DIST" -l "$LOGFILE" -s "$DOWNLOAD_DIR" -u $URL
            fi
          done
        else
            LOG "ERROR:  REPO_FILE $REPO_FILE is not a file"
        fi
    else
        LOG "ERROR:  REPO_FILE not defined $REPO_FILE"
    fi
fi

# unrpm metapackages and place files to the correct location
LOG "INFO:    Creating Repository in $REPO_DIR/$DIST"
$DO_ECHO make_testrepo.pl -c -s "$DOWNLOAD_DIR/$DIST" -t "$REPO_DIR/$DIST"

if [ x"$RESIGN" = x"true" ]; then
    LOG "INFO:    Resigning RPMs"
    # sign RPMS
    pushd "$REPO_DIR/$DIST/suse" 1>/dev/null
    $DO_ECHO rpm --resign */*.rpm
    popd  1>/dev/null
fi

# clean up to save some space
# excluded: cs da de en en_GB en_US es fi fr hu it ja nb nl pt pt_BR ru sv zh zh_CN zh_TW
if [ -d "$DELETE_PACKAGES_DIR" ]; then
    pushd "$DELETE_PACKAGES_DIR" 1>/dev/null
    LOG "INFO:    DELETING packages defined in $DELETE_PACKAGES_DIR"
    for i in *; do
        for rpmfile in $(find "$REPO_DIR/$DIST" -type f -name $i*.rpm); do
            NAME=$(rpm -qp --qf "%{NAME}" --nodigest --nosignature "$rpmfile")
            if [ "$NAME" = "$i" ]; then
                LOG "Removing $i"
                $DO_ECHO rm $i 2>$LOGFILE
            fi
        done
    done
    popd 1>/dev/null
fi

#
# packages files
#
pushd "$REPO_DIR/$DIST" 1>/dev/null
if [ "$PDB_PATH" != "" ]; then
    PDB="-p $PDB_PATH"
else
    PDB=""
fi
if [ -f "$REPO_DIR/$DIST/suse/setup/descr/EXTRA_PROV" ]; then
    EXTRA_PROV="-x $REPO_DIR/$DIST/suse/setup/descr/EXTRA_PROV"
else
    EXTRA_PROV=""
fi
if [ -n "$PDB_LANGS" ]; then
  for lang in $PDB_LANGS; do
    LANGUAGE="$LANGUAGE -l $lang"
  done
fi
LOG "INFO:    Creating package descriptions"
$DO_ECHO create_package_descr -Z -P -L -C $EXTRA_PROV $LANGUAGE $CREATE_PACKAGE_DESCR_OPTS -d suse/ $PDB -o suse/setup/descr/
popd 1>/dev/null
#
# create patterns file
#
pushd $REPO_DIR/$DIST/suse/setup/descr 1>/dev/null
LOG "INFO:    Creating patterns file"
$DO_ECHO ls *.pat* > patterns
create_directory.yast .
popd 1>/dev/null
#
# sign content and product
#
pushd $REPO_DIR/$DIST 1>/dev/null
LOG "INFO:    Creating sha1sums in content file"
$DO_ECHO create_sha1sums $CREATE_SHA1SUMS_OPTS .
#
# changelog and listings
#
LOG "INFO:    Creating Changelog, INDEX.gz and other listings"
$DO_ECHO mk_changelog .
$DO_ECHO mk_listings .
#
# other directory.yast files
#
LOG "INFO:    Creating directory.yast files"
$DO_ECHO create_directory.yast .
popd 1>/dev/null
pushd $REPO_DIR/$DIST/media.1 1>/dev/null
$DO_ECHO create_directory.yast .
popd 1>/dev/null
#
# run md5sums
#
pushd $REPO_DIR/$DIST 1>/dev/null
LOG "INFO:    Creating MD5SUM files - this may take a while"
$DO_ECHO create_md5sums .
popd 1>/dev/null

#
# make shure, everything is readable for all
#
LOG "INFO:    finally make sure everything is readable for all"
chmod -R a+r $REPO_DIR
find $REPO_DIR -type d -print0 | xargs -0 --no-run-if-empty chmod a+x
find $REPO_DIR -type f -perm -022 -print0 | xargs -0 --no-run-if-empty chmod go-w

if [ "$MKISO" = "true" ]; then
    pushd $REPO_DIR 1>/dev/null
    LOG "INFO:    Creating ISO image"
    if [ -f "$ISO_DIR/$ISO_NAME" ]; then
        LOG "WARNING: $ISO_DIR/$ISO_NAME exists: removing"
        $DO_ECHO rm "$ISO_DIR/$ISO_NAME"
        $DO_ECHO sed -i "s|.*$ISO_NAME||g" "$ISO_DIR/MD5SUMS"
    fi
    $DO_ECHO mkisofs -p "$ISO_PREPARER" -publisher "$ISO_PUBLISHER" -r -J -f -pad -A OSSEdu-$DIST -V Edu10.001 -o $ISO_DIR/$ISO_NAME $DIST/ 1>/dev/null  
    pushd $ISO_DIR 1>/dev/null
    if [ ! -f $ISO_NAME ]; then
        LOG "ERROR:  no $ISO_NAME found"
    else
        $DO_ECHO md5sum $ISO_NAME >> MD5SUMS; 
    fi
    popd 1>/dev/null
fi
if [ "$CREATE_LINKS" = "true" ]; then
    LOG "INFO:    Creating symlinks to the original package names"
    $DO_ECHO create_links.pl -r $REPO_DIR/$DIST/suse -l $LOGFILE
fi
