#!/usr/bin/perl -w
# $Id: make_testrepo.pl 35 2009-03-15 16:07:53Z lrupp $
#

BEGIN {
  $abuild_base_dir = "/usr/share/inst-source-utils";
  unshift @INC, "$abuild_base_dir/modules";
}


use strict;
use Data::Dumper;
use File::Basename;
use File::Copy;
use File::Path;
use File::Temp qw/ tempdir / ; #tempfile /;
use RPMQ;
use Needed;

my $PRGNAME="make_testrepo.pl";
my $VERSION = '0.02';
my $dist="11.0";
my $cleantargetdir=0;
my $logfile="/tmp/make_testrepo.log";
my $loglevel=2;
my $allcounter=0;
my $rpmdir="suse";
my $sourcedir="/tmp/download/repo/$dist";
my $targetdir="/tmp/download/test/$dist";
my @archs=qw(i386 i486 i586 i686 ia64 noarch ppc ppc64 s390 s390x x86_64);

sub usage {
    my $exitcode=shift || 1;
    print "\nUsage: $PRGNAME [-h] <-s sourcedir> <-t targetdir>\n";
    print "         -c           : clean target directory bevore proceed\n";
    print "         -l <logfile> : use logfile\n"; 
    print "         -h           : help\n";
    exit $exitcode;
}

sub logfile {
    my $action = shift;
    if (("$action" eq "open" ) || ("$action" eq "new" )) {
        open (LOGFILE, ">>$logfile") || warn "Couldn't open $logfile !\n";
        flock(LOGFILE,2)         || warn "Can't get lock for $logfile !\n";
        LOG("$0 started. Version $VERSION. Process ID: $$",1);
    } else {
        close LOGFILE;
    }
}

sub LOG {
  my $message = shift;
  my $level   = shift || 0;
  my $time = localtime(time);
  if ( $level <= $loglevel ) {
      print LOGFILE "[$time] [$$] $message\n";
  }
}

sub collect_rpms {
    my $searchdir=shift || "";
    my @rpmpaths=();
    push  @rpmpaths,`find $searchdir -type f -name "*.rpm" | grep -v patch.rpm | grep -v delta.rpm`;
    chomp @rpmpaths;
    return \@rpmpaths;
}

sub package_name {
  my $rpm = shift;
  $rpm =~ s/.*\///;
  return $1 if $rpm =~ /^(.*)-.*?-.*?\..*?\.rpm/;
  return $1 if $rpm =~ /^(.*-.*)\.(rpm|spm)$/;
  return $1 if $rpm =~ /^(.*)-.*?-[0-9].*?$/;
  $rpm =~ s/\..*//;
  return $rpm;
}

while  (my $param = shift (@ARGV)) {
    if (( $param eq "-h" ) || ( $param eq "--help" )){
        usage(0);
    }
    if (( $param eq "-s") || ($param eq "--sourcedir")){
        $sourcedir=shift (@ARGV);
        next;
    }
    if (( $param eq "-t") || ($param eq "--targetdir")){
        $targetdir=shift(@ARGV);
        next;
    }
    if (( $param eq "-l") || ($param eq "--logfile")){
        $logfile=shift(@ARGV);
        next;
    }
    if (( $param eq "-c") || ($param eq "--clean")){    
        $cleantargetdir=1;
        next;
    }
}

logfile("open");
die "Sourcedir not found: $!\n" if (!-d "$sourcedir");

if ($cleantargetdir){
    LOG("Cleaning $targetdir",2);
    system("rm -rf $targetdir") if (-d $targetdir);
}
mkpath("$targetdir") if (!-d $targetdir);

foreach my $arch (@archs){
    my %rpmhash=();
    next if (!-d "$sourcedir/$arch");
    my $rpmpaths=collect_rpms("$sourcedir/$arch");

    for my $file (@$rpmpaths){
        my $name=package_name($file);
        my $tmp;
        my %tmphash=RPMQ::rpmq_many($file,'NAME','VERSION','RELEASE','GROUP','ARCH', 'SOURCERPM','SIGTAG_GPG');
        $allcounter++;
        if (! $rpmhash{$name}){
            $rpmhash{$name}=\%tmphash;
            $rpmhash{$name}{'LOCATION'}=[dirname("$file")];
            $rpmhash{$name}{'RPMNAME'}=[basename("$file")];
        } else {
            $tmp=Needed::verscmp($rpmhash{$name}{'VERSION'}[0],$tmphash{'VERSION'}[0]);
            if ("$tmp" eq "-1"){
                $rpmhash{$name}=\%tmphash;
                $rpmhash{$name}{'LOCATION'}=[dirname("$file")];
                $rpmhash{$name}{'RPMNAME'}=[basename("$file")];
            } else {
                $tmp=Needed::verscmp($rpmhash{$name}{'RELEASE'}[0],$tmphash{'RELEASE'}[0]);
                if ("$tmp" eq "-1"){
                    LOG ("$name is duplicated",2);
                    $rpmhash{$name}=\%tmphash;
                    $rpmhash{$name}{'LOCATION'}=[dirname("$file")];
                    $rpmhash{$name}{'RPMNAME'}=[basename("$file")];
                }
            }
        }
    }
    LOG("Ordered: $allcounter packages for $arch",2);

    for my $pkgname (keys %rpmhash){
        if ("$rpmhash{$pkgname}{GROUP}[0]" =~ /^Metapackages/){
            LOG("Extracting Metapackage: $rpmhash{$pkgname}{NAME}[0]",2);
            my ($firstdir) = $rpmhash{$pkgname}{GROUP}[0] =~ /Metapackages\/(.*)/;
            $firstdir="CD1" if ((!defined($firstdir)) || ("$firstdir" eq ""));
            my $tmpdir=tempdir("$PRGNAME-XXXXX", CLEANUP => 1);
            system("pushd $tmpdir 1>/dev/null; unrpm -q $rpmhash{$pkgname}{LOCATION}[0]/$rpmhash{$pkgname}{RPMNAME}[0] && cp -dfr $firstdir/* $targetdir/");
        } else {
            if ((defined $rpmhash{$pkgname}{SOURCERPM}[0]) && ("$rpmhash{$pkgname}{SOURCERPM}[0]" ne "(none)")){
                my $rpmname="$rpmhash{$pkgname}{NAME}[0]-$rpmhash{$pkgname}{VERSION}[0]-$rpmhash{$pkgname}{RELEASE}[0].$rpmhash{$pkgname}{ARCH}[0].rpm";
                if (!-d "$targetdir/$rpmdir/$rpmhash{$pkgname}{ARCH}[0]"){
                    mkpath("$targetdir/$rpmdir/$rpmhash{$pkgname}{ARCH}[0]") or die "mkpath $targetdir/$rpmdir/$rpmhash{$pkgname}{ARCH}[0] failed: $!\n";
                }
                copy("$rpmhash{$pkgname}{LOCATION}[0]/$rpmhash{$pkgname}{RPMNAME}[0]","$targetdir/$rpmdir/$rpmhash{$pkgname}{ARCH}[0]/$rpmname") or die "Copy of $pkgname failed: $!\n";
                # source.rpm handling
                my $sourcerpm=`find $sourcedir -name $rpmhash{$pkgname}{SOURCERPM}[0]`;
                chomp($sourcerpm);
                if (("$sourcerpm" ne "") && (-f "$sourcerpm")){
                    my $srcdir="src";
                    if ("$sourcerpm" =~ /nosrc.rpm$/){
                        $srcdir="nosrc";
                    }
                    mkpath("$targetdir/$rpmdir/$srcdir") if (!-d "$targetdir/$rpmdir/$srcdir");
                    copy("$sourcerpm","$targetdir/$rpmdir/$srcdir/$rpmhash{$pkgname}{SOURCERPM}[0]");
                } else {
                    LOG("No src.rpm for $rpmname found",2);
                }
            }
        }
    }
}
#print Data::Dumper->Dump([\%rpmhash]);
#LOG("trying to sign $file",2);
#system("sign $file");

logfile("close");
