#!/bin/bash
# $Id: rsync_repos.sh 35 2009-03-15 16:07:53Z lrupp $
#
PATH=~/bin:$PATH
DO_ECHO=""
DIST="11.0"
DOWNLOAD_DIR="/tmp"
REMOVE_REPO_DIR="false"
REPO_DIR="/tmp/repo/$DIST"
CONFIG="/etc/sysconfig/opensuse-education"

function usage () {
    echo "Usage: `basename $0` [OPTIONS]" 
    echo "       -c <conf> : Use config instead of default $CONFIG"
    echo "       -d <dist> : Distribution version like 11.0"
    echo "       -f <file> : File containing URLs to sync (one URL per line)"
    echo "       -h        : help (this message)"
    echo "       -l <file> : Logfile"
    echo "       -r        : clean sync dir bevore start"
    echo "       -s <dir>  : dir to sync to"
    echo "       -u <url>  : URL"
    echo "       -v        : test - don't do it really, just output commands"
    echo
    exit 1
}


function syncit () {
    if [ -z "$DOWNLOAD_DIR" ]; then
        echo "DOWNLOAD_DIR not defined!"
        exit 1
    fi
    if [ -z "$DIST" ]; then
        echo "DIST not defined!"
        exit 1
    fi
    if [ -z "$URL" ]; then
        echo "URL not defined!"
        exit 1
    fi
    TMPDIR=$(mktemp -d /tmp/$DIST-XXXXXX)
    test -d "$DOWNLOAD_DIR/$DIST" || $DO_ECHO mkdir -p "$DOWNLOAD_DIR/$DIST"

    $DO_ECHO rsync $REPO_SYNC_OPTS --exclude="*debuginfo*.rpm" --exclude "*debugsource*.rpm" "$URL/" "$TMPDIR/$DIST/"
    $DO_ECHO find "$TMPDIR/$DIST/" -name "*debuginfo*rpm" -exec rm "{}" \;
    $DO_ECHO find "$TMPDIR/$DIST/" -name "*debugsource*rpm" -exec rm "{}" \;
    $DO_ECHO cp -af $TMPDIR/$DIST/* "$DOWNLOAD_DIR/$DIST/"

    rm -rf $TMPDIR
}

while getopts 'c:d:f:hl:rs:u:v' OPTION; do
    case $OPTION in
        c)
            CONFIG=$OPTARG
        ;;
        d)
            DIST1=$OPTARG
        ;;
        f)
            REPO_FILE1=$OPTARG
        ;;
        h|help)
            usage
        ;;
        l)
            LOGFILE1=$OPTARG
        ;;
        r)
            REMOVE_REPO_DIR1="true"
        ;;
        s)
            DOWNLOAD_DIR1=$OPTARG
        ;;
        u)
            URL1=$OPTARG
        ;; 
        v)
            DO_ECHO="echo"
        ;;
    esac
done
shift $(( OPTIND - 1 ))

test -f $CONFIG || {
    echo "$CONFIG is no file"
    exit 1
}
. $CONFIG
if [ -n "$DIST1" ]; then
    DIST="$DIST1"
fi
if [ -n "$REPO_FILE1" ]; then
    REPO_FILE="$REPO_FILE1"
fi
if [ -n "$LOGFILE1" ]; then
    LOGFILE="$LOGFILE1"
fi
if [ -n "$REMOVE_REPO_DIR1" ]; then
    REMOVE_REPO_DIR="$REMOVE_REPO_DIR1"
fi
if [ -n "$DOWNLOAD_DIR1" ]; then
    DOWNLOAD_DIR="$DOWNLOAD_DIR1"
fi
if [ -n "$URL1" ]; then
    URL="$URL1"
fi

if [ "$REMOVE_REPO_DIR" = "true" ]; then
    $DO_ECHO rm -rf "$DOWNLOAD_DIR/$DIST"
fi

if [ -n "$REPO_FILE" ]; then
    if [ -f "$REPO_FILE" ]; then
        URLS=$(cat "$REPO_FILE" | grep ^baseurl= | cut -d "=" -f2)
            for URL in $URLS; do
                syncit
            done
    fi
else
 if  [ -n "$URL" ]; then
    syncit
 else
    echo "No URL defined"
    exit 1
 fi 
fi
